#
#   Python GUI - Components - PyObjC
#

import AppKit
from Foundation import NSRect, NSPoint, NSSize, NSObject
from Events import Event
from GComponents import Component as GComponent

#------------------------------------------------------------------------------

#_ns_autoresizing_mask = AppKit.NSViewMinYMargin

_ns_view_to_component = {}  # Mapping from NSView to corresponding Component

#------------------------------------------------------------------------------

class Component(GComponent):

    _has_local_coords = True

    def __init__(self, _ns_view, _ns_set_autoresizing_mask = True, **kwds):
        self._ns_view = _ns_view
        _ns_view_to_component[_ns_view] = self
        GComponent.__init__(self, **kwds)

    def destroy(self):
        #print "Component.destroy:", self ###
        GComponent.destroy(self)
        _ns_view = self._ns_view
        if _ns_view in _ns_view_to_component:
            #print "Component.destroy: removing", _ns_view, "from mapping" ###
            del _ns_view_to_component[_ns_view]
        #print "Component.destroy: breaking link to", self._ns_view ###
        self._ns_view = None
    
    def get_bounds(self):
        (l, t), (w, h) = self._ns_view.frame()
        return (l, t, l + w, t + h)
    
    def set_bounds(self, (l, t, r, b)):
        ns_frame = NSRect(NSPoint(l, t), NSSize(r - l, b - t))
        self._ns_view.setFrame_(ns_frame)

    def become_target(self):
        ns_view = self._ns_view
        ns_window = ns_view.window()
        if ns_window:
            ns_window.makeFirstResponder_(ns_view)

    def _ns_mouse_down(self, ns_event, ns_view):
        self.mouse_down(self._ns_mouse_event_to_event(ns_event, ns_view))
    
    def _ns_mouse_event_to_event(self, ns_event, ns_view):
        event = Event(ns_event)
        ns_win_pos = ns_event.locationInWindow()
        #ns_view = self._ns_view
        #ns_view = self._ns_inner_view
        x, y = ns_view.convertPoint_fromView_(ns_win_pos, None)
        event.position = (x, y)
        return event

    def _ns_key_event(self, ns_event):
        event = Event(ns_event)
        self.handle(event.kind, event)

#------------------------------------------------------------------------------

class PyGUI_NSActionTarget(NSObject):
    #  A shared instance of this class is used as the target of
    #  all action messages from the NSViews of Components. It
    #  performs the action by calling the similarly-named method of
    #  the corresponding Component.
    
    def doAction_(self, ns_sender):
        self.call_method('do_action', ns_sender)
    
    def call_method(self, method_name, ns_sender):
        component = _ns_view_to_component.get(ns_sender)
        if component:
            getattr(component, method_name)()

_ns_action_target = PyGUI_NSActionTarget.alloc().init()

def ns_set_action(ns_control, method_name):
    #  Arrange for the 'action' message of the NSControl to
    #  invoke the indicated method of its associated Component.
    ns_control.setAction_(method_name)
    ns_control.setTarget_(_ns_action_target)

