#
#   Python GUI - File selection dialogs - Cocoa
#

from AppKit import NSOpenPanel, NSSavePanel, NSOKButton
from Files import FileRef

#------------------------------------------------------------------

def request_old_file(prompt = "Open File", default_dir = None, file_types = None):
    """Present a dialog requesting the name of an existing file.
    Returns a FileRef or None."""
    
    ns_panel = NSOpenPanel.openPanel()
    if prompt.endswith(":"):
        prompt = prompt[:-1]
    ns_panel.setTitle_(prompt)
    if default_dir:
        ns_dir = default_dir.path
    else:
        ns_dir = None
    if file_types:
        ns_types = [type._ns_file_type() for type in file_types]
    else:
        ns_types = None
    #print "request_old_file: ns_types =", ns_types ###
    result = ns_panel.runModalForDirectory_file_types_(ns_dir, None, ns_types)
    if result == NSOKButton:
        return FileRef(path = ns_panel.filenames()[0])
    else:
        return None

#------------------------------------------------------------------

def request_new_file(prompt = "Save File", default_dir = None,
        default_name = "", file_type = None):
    """Present a dialog requesting the name of an existing file.
    Returns a FileRef or None."""

    ns_panel = NSSavePanel.savePanel()
    if prompt.endswith(":"):
        prompt = prompt[:-1]
    if prompt.endswith(" as"):
        prompt = prompt[:-3]
    ns_panel.setTitle_(prompt)
    if default_dir:
        ns_dir = default_dir.path
    else:
        ns_dir = None
    if file_type and file_type._suffix and not file_type._mac_type:
        ns_panel.setRequiredFileType_(file_type._suffix)
    result = ns_panel.runModalForDirectory_file_(ns_dir, None)
    if result == NSOKButton:
        return FileRef(path = ns_panel.filename())
    else:
        return None
