#
#		Python GUI - View Base - PyObjC
#

import Foundation
import AppKit
import Applications
from Properties import overridable_property
from Events import Event
from GViewBases import ViewBase as GViewBase

ns_tracking_mask = (
    AppKit.NSLeftMouseDraggedMask |
    AppKit.NSRightMouseDraggedMask |
    AppKit.NSOtherMouseDraggedMask |
    AppKit.NSLeftMouseUpMask |
    AppKit.NSRightMouseUpMask |
    AppKit.NSOtherMouseUpMask)

# Need to use NSDefaultRunLoopMode here otherwise timers don't fire.
ns_tracking_mode = Foundation.NSDefaultRunLoopMode # AppKit.NSEventTrackingRunLoopMode

ns_distant_future = Foundation.NSDate.distantFuture()


class ViewBase(GViewBase):

    def _ns_track_mouse(self, ns_view):
        ns_app = Applications.ns_application
        tracking = True
        while tracking:
            ns_event = ns_app.nextEventMatchingMask_untilDate_inMode_dequeue_(
                ns_tracking_mask, ns_distant_future, ns_tracking_mode, True)
            event = self._ns_mouse_event_to_event(ns_event, ns_view)
            yield event
            tracking = event.kind <> 'mouse_up'

    def _cursor_changed(self):
        #print "ViewBase._cursor_changed:", self ###
        ns_view = self._ns_view
        ns_window = ns_view.window()
        #print "...ns_window =", ns_window ###
        if ns_window:
            #print "...invalidating cursor rects for", ns_view ###
            # invalidateCursorRectsForView_ doesn't seem to trigger
            # resetCursorRects on the view.
            #ns_window.invalidateCursorRectsForView_(ns_view)
            ns_window.resetCursorRects()

    def _ns_reset_cursor_rects(self):
        #print "ViewBase._ns_reset_cursor_rects:", self ###
        #print "...ns_view =", self._ns_view ###
        cursor = self._cursor
        if cursor:
            ns_view = self._ns_view
            ns_rect = ns_view.visibleRect()
            #print "...adding", ns_rect, "for", cursor._ns_cursor, \
            #	"with", cursor._ns_cursor.image() ###
            ns_view.addCursorRect_cursor_(ns_rect, cursor._ns_cursor)
    

class PyGUI_NS_ViewBase:
    #  Methods to be mixed in with PyGUI_NSView classes.
    
    def mouseDown_(self, ns_event):
        self.pygui_view._ns_mouse_down(ns_event, self)

    def keyDown_(self, ns_event):
        self.pygui_view._ns_key_event(ns_event)

    def keyUp_(self, ns_event):
        self.pygui_view._ns_key_event(ns_event)
    
    def becomeFirstResponder(self):
        self.pygui_view.targeted()
        return True
    
    def resignFirstResponder(self):
        self.pygui_view.untargeted()
        return True

    def resetCursorRects(self):
        self.pygui_view._ns_reset_cursor_rects()
